<?php namespace Dever\Cache;

use Dever\Config\Load as Config;
use Dever\Cache\Store;
use Dever\Http\Input;
use Dever\Debug\Process as Debug;

class Handle
{
    /**
     * instance
     *
     * @var string
     */
    static protected $instance;

    /**
     * store
     *
     * @var object
     */
    protected $store;

    /**
     * config
     *
     * @var array
     */
    protected $config;

    /**
     * expire
     *
     * @var int
     */
    protected $expire;

    /**
     * type
     *
     * @var string
     */
    protected $type;


    /**
     * getInstance
     * 
     * @return Dever\Cache\Handle;
     */
    static public function getInstance($type = 'mysql', $expire = 3600)
    {
        if(empty(self::$instance[$type]))
        {
            self::$instance[$type] = new self($expire, $type);
        }

        return self::$instance[$type];
    }

    public function __construct($expire = 3600, $type = 'mysql')
    {
        $this->expire = $expire;
        $this->type = $type;
    }

    public function store($key)
    {
        if($this->store)
        {
            return;
        }

        if(!$this->config)
        {
            $this->config = Config::$global['cache'];
        }

        $class = 'Dever\\Cache\\Store\\' . ucfirst($this->config['type']);
        $this->store = new $class();

        $this->log('connect', $this->config['type']);

        foreach($this->config['store'] as $k => $v)
        {
            if(empty($v['expire']))
            {
                $v['expire'] = $this->expire;
            }
            $this->store->connect($v);
        }
    }

    /**
     * get
     *
     * @return mixd
     */
    public function get($key)
    {
        $this->store($key);
        $param = isset($this->config['clear']) ? $this->config['clear'] : 'dever_cache';
        if(Input::get($param) == 'clear')
        {
            //$this->delete($key);
            return false;
        }
        $data = $this->store->get($key);
        //$data = json_decode(base64_decode($data), true);
        $this->log('get', $key, $data, $this->expire($key));
        return $data;
    }

    /**
     * set
     *
     * @return mixd
     */
    public function set($key, $value, $expire = 0)
    {
        $this->store($key);
        $this->init($key, $expire);
        $this->log('set', $key, $value, $expire);
        //$value = base64_encode(json_encode($value));
        return $this->store->set($key, $value, $expire);
    }

    /**
     * delete
     *
     * @return mixd
     */
    public function delete($key)
    {
        $this->store($key);
        $this->log('delete', $key, 1);
        return $this->store->delete($key);
    }

    /**
     * init
     *
     * @return mixed
     */
    protected function init($key, &$expire)
    {
        if(isset($this->config[$this->type . '_key']))
        {
            foreach($this->config[$this->type . '_key'] as $k => $v)
            {
                if(strpos($key, $k) !== false)
                {
                    $expire = $v;
                }
            }
        }

        $this->expire($key, ($expire > 0 ? $expire : $this->expire));
    }

    /**
     * expire
     *
     * @return mixed
     */
    protected function expire($key, $expire = false)
    {
        if(isset($this->config['expire']) && $this->config['expire'])
        {
            $key .= '_expire';
            if($expire > 0)
            {
                $expire = $expire*2;
                $this->store->set($key, DEVER_TIME + $expire, $expire);
            }
            else
            {
                $expire = $this->store->get($key);
                if($expire)
                {
                    $num = $expire - DEVER_TIME;
                    return '将于' . date('Y-m-d H:i:s', $expire) . '失效('.$num.'s)';
                }
            }
        }
        return false;
    }

    /**
     * log
     *
     * @return mixed
     */
    protected function log($method, $key = false, $value = false, $expire = 0)
    {
        $expire = $expire ? $expire : $this->expire;
        $log = array();
        $log['method'] = $method;
        $log['key'] = $key;
        if($value)
        {
            $log['value'] = $value;
        }
        if($expire >= 0)
        {
            $log['expire'] = $expire;
        }
        Debug::log($log, 'cache');
    }
}
