<?php namespace Dever\Data;

use Dever\Debug\Process as Debug;
use Dever\Config\Load as Config;
use Dever\Http\Output;
use Dever\Http\Input;
use Dever\Routing\Load;
use Dever\Config\Project;

class Model
{
	/**
     * database
     *
     * @var array
     */
	protected $database;

	/**
     * config
     *
     * @var array
     */
	protected $config;

	/**
     * index
     *
     * @var string
     */
	protected $index;

	/**
     * key
     *
     * @var string
     */
	protected $key;

	/**
     * db
     *
     * @var Object
     */
	protected $db;

	/**
     * __construct
     *
     * @return mixd
     */
	public function __construct($config, $index = '')
	{
		$this->config = $config;

		$this->database = Config::$global['database'];

		$this->index = $index;
	}

	/**
     * key
     *
     * @return mixd
     */
	protected function key($key = '')
	{
		$this->key = $key ? $key : (isset($this->config['project']) ? $this->config['project'] : DEVER_APP_NAME);

		return $this->key;
	}

	/**
     * db
     *
     * @return mixd
     */
	protected function db($key = '')
	{
		$key = $this->key($key);
		
		$this->checkDatabase($key);

		if(empty($this->db[$key]))
		{
			//$this->config['name'] = $key . '_' . $this->config['name'];
			
			$method = 'Dever\\Data\\' . ucwords($this->database[$key]['type']) . '\\Store';

			if(isset($this->config['cache']) && !$this->index)
			{
				$this->database[$key]['cache']['mysql'] = $this->config['cache'];
			}
			$this->db[$key] = new $method($this->database[$key]);

			if(isset($this->config['struct']))
			{
				$this->db[$key]->table($key . '_' . $this->config['name'], $this->index);
			
				$this->create($key);
			}
			else
			{
				$this->db[$key]->table($this->config['name'], $this->index, false);
			}
		}

		return $this->db[$key];
	}
	
	private function checkDatabase($key)
	{
		if(empty($this->database[$key]))
		{
			if(empty($this->database['default']))
			{
				Output::abert('core_database_exists', $key);
			}
			$this->database[$key] = $this->database['default'];
		}

		if(isset($this->database[$key][$this->index]))
		{
			$this->database[$key] = $this->database[$key][$this->index];
		}
	}
	
	private function create($key)
	{
		# 建表
		if(isset($this->config['struct']))
		{
			$this->config['type'] = isset($this->config['type']) ? $this->config['type'] : 'innodb';
			$create = $this->db[$key]->create($this->config['struct'], $this->index, $this->config['type']);
			if($create === true)
			{
				# 写入默认值
				if(isset($this->config['default']))
				{
					$this->db[$key]->inserts($this->config['default'], $this->index);
				}
			}
			else
			{
				if(isset($create['struct']))
				{
					if(count($create['struct']) < count($this->config['struct']))
					{
						$alter = array_diff_key($this->config['struct'], $create['struct']);
						if($alter)
						{
							$this->db[$key]->alter($alter, $this->config['struct'], $this->index);
						}
					}
				}
				
				# 更改表结构
				if(isset($this->config['alter']))
				{
					$this->db[$key]->alter($this->config['alter'], $this->index);
				}
			}

			# 建立索引
			if(isset($this->config['index']))
			{
				$this->index($this->config['index'], $this->index);
			}
		}
	}
	
	public function index($index)
	{
		return $this->db()->index($index);
	}

	/**
     * db
     *
     * @return mixd
     */
	public function getDb($key = '')
	{
		return $this->db($key);
	}
	
	/**
     * 获取匹配的类型
     *
     * @return mixd
     */
	private function type($method)
	{
		switch($method)
		{
			case 1:
				$method = '=';
				break;
			case 2:
				$method = 'like';
				break;
			case 3:
				$method = '>';
				break;
			case 4:
				$method = '>=';
				break;
			case 5:
				$method = '<';
				break;
			case 6:
				$method = '<=';
				break;
		}
		
		return $method;
	}
	
	/**
     * # 初始化默认方法one、list、update、insert、delete、total、all、check
     *
     * @return mixd
     */
	private function init($method, $param = array())
	{
		$array = array();
		$type = '';
		
		$this->config['request'][$method] = array
		(
			'type' => $method,
			'where' => array('id' => 'yes'),
		);
		
		# 支持单独字段更新
		$key = '';
		if(strpos($method, 'update_') !== false)
		{
			$key = str_replace('update_', '', $method);
			if($key && isset($this->config['struct'][$key]))
			{
				$match = $this->config['struct'][$key]['match'];
				if(is_array($this->config['struct'][$key]['match']))
				{
					$match = $this->config['struct'][$key]['match'][0];
				}
				$this->config['request'][$method]['type'] = 'update';
				$this->config['request'][$method]['set'][$key] = $match;
				$this->config['request'][$method]['where']['id'] = 'yes';

				return;
			}
		}

		switch($method)
		{
			case 'one':
			case 'delete':
				if(isset($param['where']))
				{
					$this->config['request'][$method]['where'] = $param['where'];
				}
				break;
			case 'updatemul':
				#批量更新操作，仅仅开启了mul_type配置或者后台可用
				if(isset(Config::$global['base']['mul_type']) && Config::$global['base']['mul_type'] > 0)
				{
					$this->config['request'][$method]['type'] = 'update';

					if(Config::$global['base']['mul_type'] == 2)
					{
						unset($this->config['request'][$method]['where']);
						$type = 'option';
					}
					else
					{
						$type = 'mul';
						$this->config['request'][$method]['where']['id'] = array('yes', 'in');
					}
				}

				break;
			case 'update':
				$type = 'set';
				$this->config['request'][$method]['input'] = false;
				break;
			case 'insert':
				$this->config['request'][$method]['input'] = false;
				unset($this->config['request'][$method]['where']);
				$type = 'add';
				break;
			case 'state':
				$this->config['request']['state'] = array
				(
					'type' => 'one',
					'where' => array
					(
						'state' => 1,
						'id' => 'yes'
					)
				);
				break;
			case 'list':
				$type = 'option';
				$this->config['request']['list'] = array
				(
					'type' => 'all',
					'order' => array('id', 'desc'),
					'page' => array(15, 'list'),
					'col' => '*|id',
					'option' => array(),
				);

				if(isset($this->config['manage']['list_type']) && $this->config['manage']['list_type'])
				{
					$this->config['request']['list']['page'][0] = 50000;
				}
				
				if(isset(Config::$global['base']['excel']))
				{
					unset($this->config['request']['list']['page']);
				}
				break;
			case 'all':
				$type = 'option';
				$this->config['request']['all'] = array
				(
					'type' => 'all',
					'order' => array('id', 'desc'),
					'col' => '*|id',
					'where' => array
					(
						'state' => 1,
					),
					'option' => array(),
					'input' => false,
				);
				break;
			case 'check':
				$type = 'option';
				$this->config['request']['check'] = array
				(
					'type' => 'one',
					'col' => '*',
					'where' => array
					(
						'state' => 1,
					),
					'order' => array('id', 'desc'),
					'option' => array(),
					'input' => false,
				);
				break;
			case 'info':
				$type = 'option';
				$this->config['request']['info'] = array
				(
					'type' => 'one',
					'col' => '*',
					'order' => array('id', 'desc'),
					'option' => array(),
					'input' => false,
				);
				break;
			case 'total':
				$type = 'option';
				$this->config['request']['total'] = array
				(
					'type' => 'count',
					'col' => 'count(1) as total',
					//'group' => array('month'),
					'option' => array(),
				);
				break;
			default:
				unset($this->config['request'][$method]);
				break;
		}

		if(strpos($this->index, 'old') !== false)
		{
			if(isset($this->config['request'][$method]['where']['state']))
			{
				unset($this->config['request'][$method]['where']['state']);
			}
			
			if(isset($this->config['request'][$method]['where']['option']))
			{
				unset($this->config['request'][$method]['where']['option']);
			}

			if(isset($this->config['request'][$method]['order']))
			{
				unset($this->config['request'][$method]['order']);
			}
		}

		if(isset($this->config['struct']) && $type)
		{
			if($type == 'option' && is_array($param))
			{
				$param['search_type'] = isset($param['search_type']) ? $param['search_type'] : Input::get('search_fulltext_type', 1);
				
				$param['search_type'] = $this->type($param['search_type']);
			}
			foreach($this->config['struct'] as $k => $v)
			{
				if(isset($v['match']))
				{
					# 这里增加value，用以区分update/select的值
					if(is_array($v['match']))
					{
						$v['value'] = $v['match'][1];
						$v['match'] = $v['match'][0];
					}
					else
					{
						$v['value'] = $v['match'];
					}
					
					# 首先匹配批量更新
					if($method == 'updatemul' && isset($v['update']) && $v['update'] == 'radio' && (is_array($v['option']) || is_object($v['option'])))
					{
						$this->config['request'][$method]['set'][$k] = $v['value'];
					}
					
					if(!isset($this->config['request'][$method][$type][$k]))
					{
						if($type == 'option')
						{
							# 位运算
							if(isset($v['bit']))
							{
								$this->config['request'][$method][$type][$k] = array('option', '&');
							}
							# 默认排序
							elseif(isset($v['order']))
							{
								if(isset($this->config['request'][$method]['order']))
								{
									if($this->config['request'][$method]['order'][0] && $this->config['request'][$method]['order'][0] != $k)
									{
										if($this->config['request'][$method]['order'][0] == 'id')
										{
											$this->config['request'][$method]['order'][0] = $k;
											$this->config['request'][$method]['order'][1] = is_string($v['order']) ? $v['order'] : 'desc';
										}
										else
										{
											$this->config['request'][$method]['order'][0] = $k . '` '.(is_string($v['order']) ? $v['order'] : 'desc').',`' . $this->config['request'][$method]['order'][0];
										}
									}
								}
								
								
							}
							# 全文检索
							if(isset($param['search_type']) && isset($v['search']) && strpos($v['search'], 'fulltext') !== false)
							{
								$this->config['request'][$method][$type][$k] = array('option', $param['search_type']);
							}

							# 时间区间
							elseif(isset($v['search']) && (strpos($v['search'], 'time') !== false || strpos($v['search'], 'date') !== false))
							{
								$this->config['request'][$method][$type]['start_' . $k] = array('yes-' . $k, '>=');
								$this->config['request'][$method][$type]['end_' . $k] = array('yes-' . $k, '<=');
							}
							elseif(isset($v['search']) && strpos($v['search'], 'mul') !== false)
							{
								$this->config['request'][$method][$type][$k] = array('option', 'like');
							}
							else
							{
								$this->config['request'][$method][$type][$k] = $v['match'];
							}

							# in
							if(isset($v['in']))
							{
								$this->config['request'][$method][$type][$k] = array('option', 'in');
							}
						}
						elseif($type != 'mul' && !isset($v['table']))
						{
							if(!empty($v['insert']) && $type == 'set')
							{
								
							}
							else
							{
								$this->config['request'][$method][$type][$k] = $v['value'];
							}
						}
					}
				}
			}
		}
		
		//print_r($_POST);die;
		//print_r($this->config['request'][$method]);
	}

	private function top($param)
	{
		if(isset($this->config['top']) && is_string($this->config['top']))
		{
			$value = isset($param[$this->config['top_key']]) ? $param[$this->config['top_key']] : Input::get($this->config['top_key']);
			if($value)
			{
				$top['value'] = $value;
			}
			else
			{
				$top = Load::get('manage/auth.getTop', array($this->config['top']));
			}

			if($top)
			{
				if(!Input::get('where_' . $this->config['top_key'])) Input::set('where_' . $this->config['top_key'], $top['value']);
				if(!Input::get('option_' . $this->config['top_key'])) Input::set('option_' . $this->config['top_key'], $top['value']);
				if(!Input::get('set_' . $this->config['top_key'])) Input::set('set_' . $this->config['top_key'], $top['value']);
				if(!Input::get('add_' . $this->config['top_key'])) Input::set('add_' . $this->config['top_key'], $top['value']);
				
				if(!isset($param['where_' . $this->config['top_key']])) $param['where_' . $this->config['top_key']] = $top['value'];
				if(!isset($param['option_' . $this->config['top_key']])) $param['option_' . $this->config['top_key']] = $top['value'];
				if(!isset($param['set_' . $this->config['top_key']])) $param['set_' . $this->config['top_key']] = $top['value'];
				if(!isset($param['add_' . $this->config['top_key']])) $param['add_' . $this->config['top_key']] = $top['value'];
			}
		}
	}

	/**
     * method
     *
     * @return mixd
     */
	public function method($method = 'one', $param = array())
	{
		if(isset($param[0]) && is_array($param[0]))
		{
			$result = array();
			foreach($param as $k => $v)
			{
				$result[] = $this->method($method, $v);
			}
			
			return $result;
		}

		if($method == 'insert' && empty($this->config['submit']))
		{
			//\Dever::submit('check', 2);
		}

		$update_state = ($method == 'update' || $method == 'insert' || strpos($method, 'update_') !== false);

		# 自动为admin赋值
		$this->updateManage($method, $update_state, $param);

		if($update_state)
		{
			$this->db();
			if(isset($this->database[$this->key]['save']))
			{
				Load::get($this->config['project'] . '/' . $this->config['name'] . '-' . $method . '-save', $param);
			}
		}

		if(!isset($this->config['request'][$method])) $this->init($method, $param);

		if(isset($param['option']))
		{
			$this->config['request'][$method]['option'] = $param['option'];
		}

		$this->diy($method, $param);
		
		$this->config['param'] = array();
		if(isset($this->config['request']) && isset($this->config['request'][$method]))
		{
			$this->hook('request', 'onload', $param);
			$this->hook($method, 'start', $param);

			if($param && !is_array($param))
			{
				if(isset($this->config['request'][$method]['where']))
				{
					$this->config['request'][$method]['where'] = array('id' => $param);
					$param = array('where_id' => $param);
				}
				else
				{
					$this->config['request'][$method]['option'] = array('id' => $param);
					$param = array('option_id' => $param);
				}
			}

			# 验证头部权限
			$this->top($param);

			if($param)
			{
				$this->config['param'] =& $param;
			}
			
			# 创建全局参数，给hook使用
			Config::$global['base']['_param'] =& $this->config['param'];

			if(isset($this->config['check']) && $update_state)
			{
				$check = false;
				if(is_array($this->config['check']))
				{
					foreach($this->config['check'] as $k => $v)
					{
						$temp = $this->check($v);
						if($temp == true)
						{
							$check = true;
						}
					}
				}
				else
				{
					$check = $this->check($this->config['check']);
				}

				if($check == true)
				{
					return;
				}
			}
			
			$this->config['response'] = $this->config['request'][$method];
			
			$this->config['input'] = true;
			
			if(isset($this->config['response']['input']))
			{
				$this->config['input'] = $this->config['response']['input'];
			}

			if(isset($this->config['response']['join']) && $this->config['response']['join'])
			{
				$this->db()->join($this->config['response']['join']);
			}

			$this->push(array('where', 'add', 'set', 'option'))->condition(array('page', 'order', 'limit', 'group'));

			$type = isset($this->config['response']['type']) ? $this->config['response']['type'] : $method;

			$this->col();

			$data = $this->db()->$type($this->config['response']['col']);

			if($data && $type == 'update' && isset($param['where_id']))
			{
				$data = $param['where_id'];
			}

			if($update_state && Project::load('manage') && isset($this->config['manage']) && isset($this->config['manage']['filter']))
			{
				$this->filter($this->config['manage']['filter'], $param, $data, $method);
			}

			if(isset($this->config['sync']) && $update_state)
			{
				$this->sync($this->config['sync'], $data);
			}

			if(isset($this->config['response']['relate']) && !$update_state && $data)
			{
				$this->relate($this->config['response']['relate'], $data, $type);
			}
			
			$this->hook($method, 'end', $data);
			
			return $data;
		}

		return array();
	}

	private function updateManage($method, $update_state, &$param)
	{
		if(Project::load('manage') && DEVER_APP_NAME == 'manage' && $this->config['project'] != 'manage')
		{
			$admin = Load::get('manage/auth.info');
			if($admin && $admin['id'] > 0)
			{
				if($method == 'insert')
				{
					$this->updateManageMethod($admin, 'insert', 'add', $param);
				}
				elseif($update_state)
				{
					$this->updateManageMethod($admin, 'update', 'set', $param);
				}
				elseif($admin['self'] == 1)
				{
					$this->updateManageMethod($admin, 'select', 'where', $param);
				}
			}
			//print_r($param);die;
		}
	}

	private function updateManageMethod($admin, $type, $prefix = 'add', &$param)
	{
		if(isset($admin['col_' . $type]) && $admin['col_' . $type] && is_array($param))
		{
			$col = explode(',', $admin['col_' . $type]);
			foreach($col as $k => $v)
			{
				if(isset($this->config['struct'][$v]) && empty($this->config['struct'][$v]['value']))
				{
					$param[$prefix . '_' . $v] = $admin['id'];
					if($prefix == 'where')
					{
						$param['option_' . $v] = $admin['id'];
					}
				}
			}
		}
	}

	private function filter($filter, $param, $data, $method)
	{
		if($method == 'update' && isset($param['where_id']))
		{
			$this->filterText($filter, $param, 'set', $param['where_id']);
		}
		elseif($method == 'insert' && $data > 0)
		{
			$this->filterText($filter, $param, 'add', $data);
		}
	}

	private function filterText($filter, $param, $prefix = 'set', $id)
	{
		$text = '';

		foreach($filter as $k => $v)
		{
			if(isset($param[$prefix . '_' . $v]))
			{
				$text .= '&dever_'.$v.'=' . $param[$prefix . '_' . $v];
			}
		}

		if($text)
		{
			$config['project'] = $this->config['project'];
			$config['table'] = $this->config['name'];
			Load::get('manage/filter.handle', array($id, $text, $config));
		}
	}

	private function check($col)
	{
		$state = true;
		if(strpos($col, '.option'))
		{
			$col = str_replace('.option', '', $col);
			$state = false;
		}

		$data = explode(',', $col);
		$id = -1;
		if(isset($this->config['param']['where_id']))
		{
			$id = $this->config['param']['where_id'];
		}
		foreach($data as $k => $v)
		{
			if(isset($this->config['struct'][$v]))
			{
				if(isset($this->config['param']['set_' . $v]))
				{
					$param['option_' . $v] = $this->config['param']['set_' . $v];
				}
				elseif(isset($this->config['param']['add_' . $v]))
				{
					$param['option_' . $v] = $this->config['param']['add_' . $v];
				}

				$temp = explode('-', $this->config['struct'][$v]['name']);
				$name[] = $temp[0];
			}
		}

		if(isset($param) && $param)
		{
			$info = Load::get($this->config['project'] . '/' . $this->config['name'] . '-info', $param);

			if($id > 0 && $info && $info['id'] != $id)
	        {
	        	if($state == true)
	        	{
	        		Output::abert(implode(',', $name).'已经存在');
	        	}
	        	else
	        	{
	        		return true;
	        	}
	        }
	        elseif($id < 0 && $info)
	        {
	            if($state == true)
	        	{
	        		Output::abert(implode(',', $name).'已经存在');
	        	}
	        	else
	        	{
	        		return true;
	        	}
	        }
		}

		return false;
	}

	private function relate($config, &$data, $type)
	{
		if($type == 'all')
		{
			foreach($data as $k => $v)
			{
				$this->relate($config, $data[$k], 'one');
			}
		}
		else
		{
			foreach($config as $k => $v)
			{
				foreach($v as $i => $j)
				{
					$v[$i] = $data[$j];
				}
				$data[$k] = Load::get($k, $v);
			}
		}
	}

	private function sync($config, $id)
	{
		foreach($config as $k => $v)
		{
			$id = $id > 0 ? $id : $this->config['param']['where_id'];

			$info = Load::get($this->config['project'] . '/' . $this->config['name'] . '-one', $id);
			if(empty($info[$v['where'][1]]))
			{
				break;
			}

			if($v['type'] == 'delete')
			{
				Load::get($k . '-delete', array
				(
					'where' => array
					(
						$v['where'][0] => 'yes',
					),
					'where_' . $v['where'][0] => $info[$v['where'][1]],
				));
			}
			foreach($v['update'] as $i => $j)
			{
				if(strpos($i, '-'))
				{
					$t = explode('-', $i);
					$i = $t[0];
				}
				$value = $info[$j];

				if($value)
				{
					$value = explode(',', $value);
					foreach($value as $a => $b)
					{
						$method = 'insert';
						$type = 'add';
						$param = array();
						if($v['type'] != 'delete')
						{
							$check = Load::get($k . '-info', array
							(
								'option_' . $i => $b, 
								'option_' . $v['where'][0] => $info[$v['where'][1]],
							));

							if($check)
							{
								$method = 'update';
								$type = 'set';
								$param = array
								(
									'where_id' => $check['id'],
								);
							}
						}

						if($method)
						{
							$param += array
							(
								$type . '_' . $i => $b, 
								$type . '_' . $v['where'][0] => $info[$v['where'][1]],
							);

							if(isset($v['sync']))
							{
								foreach($v['sync'] as $c => $d)
								{
									if(isset($info[$d]) && $info[$d])
									{
										$param[$type . '_' . $c] = $info[$d];
									}
								}
							}

							Load::get($k . '-' . $method, $param);
						}
					}
				}
			}
		}
	}
	
	private function col()
	{
		if(empty($this->config['response']['col']))
		{
			$this->config['response']['col'] = '*';
		}

		if(isset(Config::$global['base']['sql']) && Config::$global['base']['sql'] && strpos($this->config['response']['col'], '*') !== false)
		{
			$temp = array();
			foreach($this->config['struct'] as $k => $v)
			{
				if(isset($v['type']))
				{
					$temp[$k] = '`' . $k . '`';
				}
			}
			
			$this->config['response']['col'] = str_replace('*', implode(',', $temp), $this->config['response']['col']);
		}
	}

	private function diy($method, $param)
	{
		if(($method == 'all' || $method == 'list') && is_array($param))
		{
			#diy all
			$diy = array('page', 'group', 'order', 'col', 'limit', 'relate', 'join');
			foreach($diy as $k => $v)
			{
				if(isset($param[$v]))
				{
					$this->config['request'][$method][$v] = $param[$v];
				}
			}

			$diy = array('where', 'option');
			foreach($diy as $k => $v)
			{
				if(isset($param[$v]))
				{
					$this->config['request'][$method][$v] = array_merge($this->config['request'][$method][$v], $param[$v]);
				}
			}
		}
	}

	/**
     * hook 插件钩子处理
     * @param string $key
     *
     * @return mixd
     */
	private function hook($key, $method = 'start', &$param)
	{
		# 不再继续执行hook，就设置这个参数
		if(Input::get('dever_onload') == 'no')
		{
			if($method == 'end')
			{
				//Input::set('dever_onload', 'yes');
			}
		}
		elseif(isset($this->config[$method][$key]))
		{
			Input::set('dever_onload', 'no');
			# 如果定义了top，就要传过去
			if(isset($this->config['top']) && is_array($this->config['top']))
			{
				$param = array($param, $this->config['top']);
			}
			# 如果定义了auth，就要传过去
			if(isset($this->config['auth']) && is_array($this->config['auth']))
			{
				$param = array($param, $this->config['auth']);
			}
			if(is_array($this->config[$method][$key]))
			{
				foreach($this->config[$method][$key] as $k => $v)
				{
					$data = Load::get($v, $param);
				}
				if($data)
				{
					$param = $data;
				}
			}
			else
			{
				Input::set('dever_onload', 'no');
				$data = Load::get($this->config[$method][$key], $param);

				if($data)
				{
					$param = $data;
				}
			}
		}
	}

	/**
     * push
     *
     * @return mixd
     */
	private function push($param)
	{
		foreach($param as $k => $v)
		{
			if(isset($this->config['response'][$v]) && is_array($this->config['response'][$v]))
			{
				# 验证传入参数与配置是否相同
				/*
				$request = Input::prefix($v . '_');
				if($v == 'where')
				{
					$request = Input::prefix('where_');
				}
				*/
				
				$value = array();
				foreach($this->config['response'][$v] as $i => $j)
				{
					$t = array();
					if(is_array($j))
					{
						$t = $j;
						$j = $t[0];
					}
					$temp = $this->request($v. '_' . $i, $j, '', $i, $v);
					//if($temp || (($v == 'add' || $v == 'set') && $temp && $temp == 0))
					if($temp || ($temp === '0' || $temp === 0))
					//if($temp)
					{
						if(is_array($temp))
						{
							if(isset($this->config['struct'][$i]) && isset($this->config['struct'][$i]['bit']))
							{
								$vt = 0;
								foreach($temp as $ki => $vi)
								{
									if(isset($this->config['struct'][$i]['bit'][$vi]))
									{
										$vt += $this->config['struct'][$i]['bit'][$vi];
									}
								}
								$temp = $vt;
							}
							elseif(isset($temp[0]) && is_array($temp[0]))
							{
								$temp = base64_encode(json_encode($temp));
							}
							else
							{
								$temp = str_replace(',0', '', implode(',', $temp));
							}
						}
						
						if($temp === 'null')
						{
							$temp = '';
						}
						$g = array($i, $temp);
						if(isset($t[1]))
						{
							$g[2] = $t[1];
						}
						if(isset($t[2]))
						{
							$g[3] = $t[2];
						}
						$value[] = $g;
					}
				}

				if($value)
				{
					if($v == 'option')
					{
						$v = 'where';
					}
					
					$this->opt($v, $value);

					$this->db()->$v($value);
				}
			}
		}

		return $this;
	}
	
	/**
     * opt
     *
     * @return mixd
     */
	private function opt($method, $param)
	{
		if(Project::load('manage') && isset(Config::$global['base']['opt']) && Config::$global['base']['opt'] && $this->config['project'] . $this->config['name']  != 'manageopt')
		{
			if($method == 'where')
			{
				$col = array();
				foreach($param as $k => $v)
				{
					$col[] = $v[0];
				}
			}
			
			elseif($method == 'order')
			{
				$col = $param[0];
				if(is_string($col))
				{
					if(strpos($col, '`') !== false)
					{
						$col = str_replace(array('`', 'desc', 'asc', ' '), '', $col);
					}
					
					$col = explode(',', $col);
				}
				elseif(is_array($col))
				{
					foreach($col as $k => $v)
					{
						$col[] = $k;
					}
				}
			}
			
			if(isset($col) && $col)
			{
				Opt::push($this->config['project'], $this->config['name'], $col);
			}
		}
	}

	/**
     * condition
     *
     * @return mixd
     */
	private function condition($param)
	{
		foreach($param as $k => $v)
		{
			if(isset($this->config['response'][$v]) && $this->config['response'][$v])
			{
				if($v == 'page')
				{
					$value = $this->config['response'][$v];

					if(isset($this->config['param']['page']) && $this->config['param']['page'] != $value)
					{
						$value[1] = array_merge($value[1], $this->config['param']['page']);
					}
					
					if(is_string($value[1]))
					{
						$temp[] = $value[1];
						unset($value[1]);
						$value[1] = $temp;
					}

					if(isset($value[2])) $value[1][2] = $value[2];
				}
				else
				{
					$temp = '';
					$value = $this->request($v, $this->config['response'][$v], '-', $temp);

					if(!$value)
					{
						$value = $this->config['response'][$v];
					}
				}

				if(is_string($value))
				{
					$value = array($value, '');
				}

				if(empty($value[0])) $value[0] = $value;
				if(empty($value[1])) $value[1] = '';

				$this->opt($v, $value);

				$this->db()->$v($value[0], $value[1]);
			}
		}

		return $this;
	}

	/**
     * request
     *
     * @return mixd
     */
	private function request($key, $value, $split = '', &$index, $method = '')
	{
		//$index = str_replace(array('add_', 'where_', 'set_', 'option_'), '', $key);
		
		if($index && strpos($value, 'yes-') !== false)
		{
			$temp = explode('-', $value);
			$value = $temp[0];
			$index = $temp[1];
		}
		
		if($index && $value == 'yes')
		{
			$value = 'option';
			if(strpos($index, '.') !== false)
			{
				//$temp = explode('.', $index);
				//$index = $temp[1];
			}
			if(isset($this->config['struct'][$index]) && is_array($this->config['struct'][$index]) && isset($this->config['struct'][$index]['match']) && $this->config['struct'][$index]['match'])
			{
				if(is_array($this->config['struct'][$index]['match']))
				{
					$value = $this->config['struct'][$index]['match'][0];
				}
				else
				{
					$value = $this->config['struct'][$index]['match'];
				}
			}
		}
		$state 		= false;
		
		if(isset($this->config['param'][$key]))
		{
			$request 	= $this->config['param'][$key];
		}

		

		if($value == 'option')
		{
			$value = '';
			$state = true;
		}

		$callback = is_string($value) && function_exists($value);
		if($callback)
		{
			$callback = $value;
			$value = '';
		}

		

		if(isset($request) && ($request === '0' || $request === 0))
		{
			return 0;
		}


		
		if(empty($request))
		{	
			if($this->config['input'])
			{
				$request 	= Input::get($key, $value);
			}
			else
			{
				$request = $value;
			}
		}



		/*
		if(is_array($request))
		{
			$request = implode(',', $request);
		}
		*/
		if(is_string($value) && strpos($value, '/') === 0)
		{
			$state = preg_match($value, $request);
		}
		elseif(!empty($request))
		{
			if($callback)
			{
				$state = $callback($request);
			}
			elseif(is_string($request) && $split && strpos($request, $split) !== false)
			{
				$request = explode($split, $request);
			}

			$state = true;
		}

		if($key == 'set_name')
		{
			//echo $state;die;
		}

		//Debug::log(array('text' => 'model', 'state' => $state, 'preg' => $value, 'key' => $key, 'value' => $request));


		if($state)
		{
			if(empty(Config::$global['base']['model_callback']) && $index && isset($this->config['struct'][$index]) && is_array($this->config['struct'][$index]) && isset($this->config['struct'][$index]['callback']) && $this->config['struct'][$index]['callback'] && $request)
			{
				$call = $this->config['struct'][$index]['callback'];
				if($call == 'maketime')
				{
					if(is_string($request))
					{
						$request = \Dever::maketime($request);
					}
				}
				else
				{
					if(strpos($call, '.'))
					{
						$temp = explode('.', $call);
						$call = $temp[0];
						$request = $call($temp[1], $request);
					}
					else
					{
						$request = $call($request);
					}
				}
			}

			if(($method == 'set' || $method == 'add') && isset($this->config['struct'][$index]['update']))
			{
				if(is_string($request) && empty($this->config['struct'][$index]['strip']))
				{
					$request = $this->safe_replace($request);
					//$request = strip_tags($request);
					//$request = htmlspecialchars($request);
				}

				if(isset($this->config['struct'][$index]['key']) && isset(Config::$global['host']['image']) && strpos($request, Config::$global['host']['image']) !== false)
				{
					$request = str_replace(Config::$global['host']['image'], '', $request);
				}
			}
			
			return $request;
		}

		# error
		if($method != 'option' && $method != 'add' && $method != 'set')
		{
			if(isset($this->config['struct'][$index]['desc']) && $this->config['struct'][$index]['desc'])
			{
				Output::abert($this->config['struct'][$index]['desc']);
			}
			else
			{
				Output::abert('core_database_request', array($key, ($value ? $value : $callback)));
			}
		}

		return false;
	}
	
	private function safe_replace($content)
	{
		$tags = array(

			"'<iframe[^>]*?>.*?</iframe>'is",

			"'<frame[^>]*?>.*?</frame>'is",

			"'<script[^>]*?>.*?</script>'is",

			"'<head[^>]*?>.*?</head>'is",

			"'<title[^>]*?>.*?</title>'is",

			"'<meta[^>]*?>'is",

			"'<link[^>]*?>'is",
		);

		return preg_replace($tags, "", $content);
	}
}
