<?php namespace Dever\Data;

use Dever;
use Dever\Data\Model;
use Dever\Http\Input;
use Dever\Page\Main as Page;

class Service
{
    protected $model;
    
    protected $db;

    public function __construct()
    {
        $config['name'] = 'table';
        $config['project'] = DEVER_APP_NAME;
		$this->model = new Model($config, 'service_' . DEVER_APP_NAME);
        $this->db = $this->model->getDb();
    }

    public function query($sql, $value = array())
    {
        return $this->db->exe($sql, $value);
    }

    /**
     * @desc 执行SQL语句
     * @param  $sql 要执行的sql语句
     * @param  $value 预处理要绑定的数据
     * @param  $page 是否进行分页 参数：template 分页模板名称 maxnum 显示的记录数 maxpage 显示的最大页数 current 当前页数
     * @param  $cache 是否进行缓存，如缓存，则为缓存的key
     * @return array
     */
    public function all($sql, $value = array(), $page = array(), $cache = false)
    {
        # 这里的缓存是mysql缓存
        if($cache)
        {
            $data = $this->db->cache($cache);
            if($data)
            {
                return $data;
            }
        }

        if($page)
        {
            $data = $this->page($page, $sql);
        }
        else
        {
            $data = $this->query($sql, $value)->fetchAll();
        }
        
        if($cache)
        {
            $this->db->cache($cache, $data);
        }

        return $data;
    }

    /**
     * cache 数据业务缓存
     *
     * @return object
     */
    public function cache($key = false, $data = array())
    {
        return Dever::cache($key, $data);
    }

    /**
     * log 调试工具
     *
     * @return object
     */
    public function log($param)
    {
        return Debug::log($param);
    }

    /**
     * page
     *
     * @return object
     */
    public function page($config = array(), $sql = false)
    {
        empty($config['template']) && $config['template'] = 'list';

        empty($config['key']) && $config['key'] = 'current';

        empty($config['link']) && $config['link'] = '';

        empty($config['num']) && $config['num'] = 10;

        $page = Page::getInstance($config['key']);

        $page->template($config['template']);

        $page->link($config['link']);

        $total = Input::get('pt', -1);

        if(isset($config['explode']) && isset($config['content']))
        {
            $content = explode($config['explode'], $config['content']);
            $page->offset(1);
            $data = $page->data($content, $total);
        }
        else
        {
            $data = $page->count($sql, $page->offset($config['num']), $total, $this);
        }
        
        return $data;
    }
}