<?php namespace Dever\Data;

use Dever\File\Path;
use Dever\Page\Main as Page;
use Dever\Config\Load as Config;
use Dever\Cache\Handle;
use Dever\Debug\Process as Debug;
use Dever\Http\Input;

class Store
{
	/**
     * read
     *
     * @var Dever\Data\Connect
     */
    protected $read;

    /**
     * update
     *
     * @var Dever\Data\Connect
     */
    protected $update;
    
    /**
     * config
     *
     * @var array
     */
    protected $config;

    /**
     * table
     *
     * @var string
     */
	protected $table;

    /**
     * value
     *
     * @var array
     */
    protected $value = array();

	/**
     * instance
     *
     * @var string
     */
    static protected $instance;

    /**
     * getInstance
     * 
     * @return Dever\Data\Store;
     */
    static public function getInstance($config)
    {
        if(empty(self::$instance))
        {
            self::$instance = new self();
        }

        self::$instance->register($config);

        return self::$instance;
    }

    /**
     * __construct
     *
     * @return mixd
     */
    public function __construct($config)
    {
    	$this->init();

		$this->register($config);
    }

    /**
     * register
     *
     * @return mixd
     */
	private function register($config)
	{
        # read update 
        if(is_array($config['host']))
        {
            $host = $config['host'];

            $config['host'] = $host['read'];

            $this->read = $this->connect($config);

            $config['host'] = $host['update'];

            $this->update = $this->connect($config);
        }
        elseif(is_string($config['host']) && strpos($config['host'], ',') !== false)
        {
            $host = explode(',', $config['host']);

            $config['host'] = $host[0];

            $this->read = $this->connect($config);

            $config['host'] = $host[1];

            $this->update = $this->connect($config);
        }
        else
        {
            $this->read = $this->update = $this->connect($config);
        }
        
        $this->config = $config;
	}

	/**
     * table
     *
     * @return object
     */
	public function table($table, $name = '', $state = true)
	{
        if(defined('DEVER_DB_PREFIX'))
        {
            $table = DEVER_DB_PREFIX . '_' . $table;
        }
		elseif($state == true && defined('DEVER_PROJECT') && DEVER_PROJECT != 'default')
		{
			$table = DEVER_PROJECT . '_' . $table;
		}
		
        $this->getTable($table);

        if($state == true && isset($this->sql) && isset($global['base']['sql']) && Config::$global['base']['sql'])
        {
            $file = $this->file($name);
            
            if(is_file($file))
            {
                $config = include($file);
                $this->sql->setColOrder($config['order']);
            }
        }

        return $this;
	}

	/**
     * table
     *
     * @return object
     */
	public function index($index, $name = '')
	{
		if(empty($index))
		{
			return false;
		}
		$file = $this->file($name);
        if(!is_file($file))
        {
            return false;
        }
        
        $data = include($file);
        
        if(isset($index['version']))
        {
			$version = $index['version'];
			
			unset($index['version']);
		}
		else
		{
			$version = 1;
		}
        
        if(empty($data['index']) || (isset($data['index']) && $data['index'] < $version))
        {
        	$data = $this->getIndex($version, $index);

        	file_put_contents($file, '<?php return ' . var_export($data, true) . ';');
        }

        return $this;
	}


	/**
     * alter table
     *
     * @return mixed
     */
    public function alter($alter, $struct = array(), $name = '')
    {
    	if(empty($this->sql))
        {
            return false;
        }
        if(empty($alter))
        {
            return false;
        }
        $file = $this->file($name);

        if(!is_file($file))
        {
            return false;
        }
        
        $data = include($file);
        if(is_array($struct))
        {
            $sql = $this->sql->alter($this->table, $alter);

            $this->exe($sql);

            $this->log($sql, 'alter');
            
            $data['struct'] = array_flip(array_keys($struct));

            file_put_contents($file, '<?php return ' . var_export($data, true) . ';');
        }
        else
        {
            if(isset($alter['version']))
            {
                $version = $alter['version'];
            }
            else
            {
                $version = 1;
            }
            
            if(isset($alter[$version]) && (empty($data['alter']) || (isset($data['alter']) && $data['alter'] != $version)))
            {
                $sql = $this->sql->alter($this->table, $alter[$version]);

                $this->exe($sql);

                $this->log($sql, 'alter');
                
                $data['alter'] = $version;

                file_put_contents($file, '<?php return ' . var_export($data, true) . ';');
            }
        }
        

        return true;
    }

    /**
     * insert the default value
     *
     * @return mixed
     */
	public function inserts($value, $name = '')
	{
		$file = $this->file($name);
        if(!is_file($file))
        {
            return false;
        }

        $data = include($file);
		if(isset($value['col']) && isset($value['value']))
		{
            $this->truncate();

			$data = $this->getInserts($value, $data);

			file_put_contents($file, '<?php return ' . var_export($data, true) . ';');
		}

        return true;
	}

	/**
     * file
     *
     * @return mixed
     */
	public function file($name = '')
	{
        if(isset(Config::$global['base']['data']))
        {
            $path = Config::$global['base']['data'];
        }
        else
        {
            $path = DEVER_PATH . 'data/';
        }

        if($name)
        {
        	$name = $this->table . '_' . $name . '.php';
        }
        else
        {
        	$name = $this->table . '.php';
        }

        $temp = explode('_', $this->table);

		$file = Path::create($path . 'database/', $temp[0] . '/' . $name);
		return $file;
	}

	/**
     * create
     *
     * @return mixed
     */
	public function create($struct, $name = '', $type = 'innodb')
	{
        if(isset(Config::$global['base']['create']))
        {
            return false;
        }
        $file = $this->file($name);
        if(is_file($file))
        {
            return include($file);
        }

        if(isset($this->sql))
        {
        	$sql = $this->sql->create($this->table, $struct, $type);

        	$this->exe($sql);

        	$this->log($sql, 'create');
        }
        
        $data['time'] = DEVER_TIME;
        
        $data['table'] = $this->table;
        
        if(isset($this->sql))
        {
        	$data['create'] = $sql;

        	$data['struct'] = array_flip(array_keys($struct));
        }
        else
        {
        	$data['create'] = $struct;

        	$this->log($data, 'create');
        }

        file_put_contents($file, '<?php return ' . var_export($data, true) . ';');
        
        return true;
	}

    private function truncate()
    {
        if(isset($this->sql))
        {
            $sql = $this->sql->truncate($this->table);

            $this->exe($sql);

            $this->log($sql, 'truncate');
        }
    }

	/**
     * page
     *
     * @return object
     */
    public function page($num, $config = array())
    {
        $this->reset('limit');

        empty($config[0]) && $config[0] = 'list';

        empty($config[1]) && $config[1] = 'current';

        empty($config[2]) && $config[2] = '';

        $page = Page::getInstance($config[1]);

        $page->template($config[0]);

        $page->link($config[2]);

        $total = Input::get('pt', -1);

        if($total < 0)
        {
            $total = $this->count('clear');
        }

        $page->total($total);
        
        $this->limit($num, $page->offset($num));

        return $this;
    }


    public function cache($key = false, $data = array())
    {
        $cache = isset($this->config['cache']) ? $this->config['cache'] : Config::$global['cache'];
        if(empty($cache['mysql']))
        {
            return;
        }

        $handle = Handle::getInstance('mysql', $cache['mysql']);
        if(!$key)
        {
            $keys = $handle->get($this->table);
            if($keys)
            {
                foreach($keys as $k => $v)
                {
                    $handle->delete($k);
                }
            }
        }
        if($data)
        {
            $keys = $handle->get($this->table);
            $keys[$key] = 1;

            $handle->set($this->table, $keys);

            return $handle->set($key, $data);
        }
        return $handle->get($key);
    }

    /**
     * error
     *
     * @return error
     */
    public function error($msg, $sql = '')
    {
        if(isset($this->sql))
        {
            if($sql)
            {
                $msg = array('sql' => $sql, 'error' => $msg);
            }
            elseif(is_object($msg))
            {
                $msg = (array) $msg;
            }
            elseif(is_string($msg))
            {
                $msg = array('sql' => $msg);
            }
            Debug::wait($msg, 'Dever SQL DB Error!');
        }
        else
        {
            if(is_string($msg))
            {
                $msg = array('value' => $msg);
            }
            Debug::wait($msg, 'Dever NOSQL DB Error!');
        }
    }

    /**
     * log
     *
     * @return log
     */
    public function log($value, $param, $data = array())
    {
    	if(isset($this->sql))
    	{
    		Debug::log(array('sql' => $value, 'value' => $param, 'data' => $data), 'db');
    	}
    	else
    	{
    		Debug::log(array('value' => $value), 'db');
    	}
    }
}