<?php namespace Dever\Debug;

use Dever;
use Dever\Config\Load as Config;
use Dever\Http\Input;
use Dever\Http\Output;

class Process
{
	static private $data;

	static private $tool;

	static private $time;

	static private $memory;

	static public function init()
	{
		if(isset(Config::$global['debug']['request']) && Config::$global['debug']['request'])
		{
			$request = Input::get(Config::$global['debug']['shell']) == 'request';
			if(is_string(Config::$global['debug']['request']))
			{
				if(strpos(Config::$global['debug']['request'], Dever::ip()) !== false)
				{
					if($request)
					{
						return true;
					}
				}
			}
			elseif(Config::$global['debug']['request'] === 2 || $request)
			{
				return true;
			}
		}
		return false;
	}

	static public function report()
	{
		ini_set('display_errors', true);
		self::tool();
	}

	static private function tool()
	{
		if(self::$tool)
		{
			return true;
		}
		if(class_exists('\\Whoops\\Run'))
		{
			self::$tool = new \Whoops\Run;
			self::$tool->pushHandler(new \Whoops\Handler\PrettyPageHandler);
			if(\Whoops\Util\Misc::isAjaxRequest())
			{
			    self::$tool->pushHandler(new \Whoops\Handler\JsonResponseHandler);
			}

			self::$tool->register();

			return true;
		}

		return false;
	}

	static public function log($msg, $type = 'log')
	{
		if(self::init())
		//if(true)
		{
			if(isset($msg['value']) && isset($msg['sql']) && is_array($msg['value']))
			{
				foreach($msg['value'] as $k => $v)
				{
					$msg['sql'] = str_replace($k, $v, $msg['sql']);
				}
				unset($msg['value']);
			}

			$msg = Output::export(array('msg' => $msg, 'time' => array('current' => self::time(2), 'total' => self::time()),'memory' => array('current' => self::memory(2), 'total' => self::memory())), false, true);
			
			self::add($type, $msg);
		}
	}

	static public function runtime()
	{
		if(self::init())
		{
			self::add('runtime', Output::export(self::loadfile(),false, true), 'Load Files');

			$msg = Output::export(array('time' => self::time(), 'memory' => self::memory()), false, true);

	        self::add('runtime', $msg, 'Total');
		}
	}

	static private function time($state = 1)
	{
		$start = DEVER_START;
		if($state == 2 && self::$time)
		{
			$start = self::$time;
		}
		list($a, $b) = explode(' ', $start); 
        $s = ((float)$a + (float)$b);

        self::$time = microtime();

		list($a, $b) = explode(' ', self::$time); 
        $e = ((float)$a + (float)$b);

        return '[' . ($e - $s) . 'S]';
	}

	static private function memory($state = 1)
	{
		$memory = memory_get_usage();
		if($state == 2 && self::$memory)
		{
			$memory = $memory - self::$memory;
		}
		self::$memory = $memory;
		
        return '[' . ($memory/1024) . 'KB]';
	}

	static private function loadfile()
	{
        return get_included_files();
	}

	static private function add($method, $msg, $key = false)
	{
		if($key)
		{
			self::$data[$method][$key] = $msg;
		}
		else
		{
			self::$data[$method][] = $msg;
		}
	}

	static public function wait($msg = '', $notice = 'Dever Data Debug!')
	{
		if($msg && self::tool())
		{
			$handler = self::$tool->getHandlers();
			if(self::$data && is_array(self::$data))
			{
				foreach(self::$data as $k => $v)
				{
					$handler[0]->addDataTable(ucwords($k), $v);
				}
			}
			if(is_array($msg))
			{
				$msg = array('array' => Output::export($msg, false, true));
			}
			$handler[0]->addDataTable('Data', $msg);
			throw new \RuntimeException($notice);
		}
		else
		{
			print_r($msg);
		}
		die;
	}

	static public function data()
	{
		if(self::init())
		{
			self::out();
		}
	}

	static public function out()
	{
		self::runtime();

		if(self::$data && is_array(self::$data))
		{
			if(self::tool())
			{
				$handler = self::$tool->getHandlers();

				foreach(self::$data as $k => $v)
				{
					$handler[0]->addDataTable(ucwords($k), $v);
				}
				throw new \RuntimeException("Dever Runtime Debug!");
			}
			else
			{
				echo self::html();
			}
		}
	}

	static private function html($show = 'display:none;')
	{
		self::runtime();

		$fix = 'fixed';
		if(!$show)
		{
			$fix = '';
		}
		$html = '<div style="position:'.$fix.';z-index:10000;bottom:0;background:white;overflow:auto;width:100%;height:auto;">';
		
		if(self::$data)
		{
			foreach(self::$data as $k => $v)
			{
				$html .= '<a style="font-size:14px;font-weight:bold;margin-left:5px;" href="javascript:;" onclick="var a = $(this).next();if(a.get(0).style.display == \'none\'){a.show();$(this).parent().height(500)}else if(a.get(0).style.display != \'none\'){a.hide();$(this).parent().height(\'auto\')}">' . $k . '</a>';
				
				$html .= '<div style="'.$show.'">';
				$html .= '<table border="1" style="width:100%;">';
				
				foreach($v as $i => $j)
				{
					$html .= '<tr>';
					
					$html .= '<td>' . $j . '</td>';
					
					$html .= '</tr>';
				}
				
				$html .= '</table>';
				
				$html .= '</div>';
			}
		}

		$html .= '</div>';
		
		return $html;
	}
}