<?php namespace Dever\Routing;

use Dever;
use Dever\Data\Model;
use Dever\Http\Output;
use Dever\Http\Input;
use Dever\Config\Project;
use Dever\Security\Api;
use Dever\Config\Load as Config;
use Dever\Debug\Process as Debug;
use Dever\Routing\Step;
use Dever\Routing\Uri;

class Load
{
    /**
     * database 定义database目录
     *
     * @var string
     */
    const DATABASE =  'database/';

    /**
     * library 定义library目录
     *
     * @var string
     */
    const LIBRARY =  'src/';

    /**
     * service 定义service目录
     *
     * @var string
     */
    const SERVICE =  'service/';

    /**
     * NAME 定义类库开发的命名空间
     *
     * @var string
     */
    const NAME =  'DeverApp\\';

    /**
     * SERVER_NAME 定义业务类的命名空间
     *
     * @var string
     */
    const SERVER_NAME =  'DeverService\\';

    /**
     * config
     *
     * @var array
     */
    protected $config;

    /**
     * setting
     *
     * @var array
     */
    protected $setting;

    /**
     * class
     *
     * @var array
     */
    protected $class;

    /**
     * data
     *
     * @var array
     */
    protected $data;

    /**
     * param
     *
     * @var array
     */
    protected $param;

    /**
     * path
     *
     * @var string
     */
    protected $path;

    /**
     * attr
     *
     * @var string
     */
    protected $attr;

    /**
     * load
     *
     * @var array
     */
    protected $load;

    /**
     * instance
     *
     * @var string
     */
    static protected $instance;

    /**
     * get
     *
     * @param  string  $method
     * @param  array  $param
     * @return \Dever\Routing\Loader
     */
    static public function get($method, $param = array())
    {
        if(empty(self::$instance))
        {
            self::$instance = new self();
            spl_autoload_register(array(self::$instance, 'autoload'));
        }

        if(strpos($method, '?'))
        {
            $temp = explode('?', $method);
            $method = $temp[0];
            parse_str($temp[1], $param);
        }

        return self::$instance->load($method, $param);
    }

    /**
     * load file
     *
     * @param  string  $method
     * @param  array  $param
     * @return \Dever\Routing\Loader
     */
    static public function object($method, $param = array())
    {
        return (object) self::get($method, $param);
    }

    /**
     * autoload
     *
     * @return mixed
     */
    private function autoload($class)
    {
        $temp = explode('\\', $class);
        if(strpos($class, self::NAME) !== false && isset($temp[1]))
        {
            $project = Project::load(strtolower($temp[1]));
            if($project)
            {
                $file = $project['path'] . self::LIBRARY . str_replace($temp[0] . '/' . $temp[1] . '/', '', str_replace('\\', '/', $class)) . '.php';
                if(is_file($file))
                {
                    $this->file($file);
                }
                else
                {
                    Output::abert('file_exists', $file);
                }
            }
        }
    }

    /**
     * setup 如果需要调取其他项目里的类，必须通过该方式
     *
     * @return mixed
     */
    private function load($key, $param = array())
    {
        //Config::$global['load'][$key] = true;
        //$this->initParam($param);

        $state = false;

        $attr = '';
        if(strpos($key, '#') !== false)
        {
            $temp = explode('#', $key);

            $key = $temp[0];

            $attr = $temp[1];
        }

        if(isset($param['cache']) && $param['cache'])
        {
            $state = true;
        }
        elseif(isset($this->param[$key]) && $this->param[$key] != $param)
        {
            $state = true;
        }

        $this->param[$key] = $param;
        
        unset($param);

        # 增加缓存
        $this->data[$key] = $this->cache($key);

        if(!$this->data[$key])
        {
            if(empty($this->config[$key]))
            {
                $this->config($key, $state);
            }

            if($state == true || empty($this->data[$key]))
            {
                $this->import(array('model','library','service'), $key);
                //$this->path = false;
            }

            $this->cache($key, $this->data[$key]);
        }

        if($attr)
        {
            if(isset($this->data[$key][$attr]))
            {
                return $this->data[$key][$attr];
            }
            return false;
        }

        return $this->data[$key];
    }

    /**
     * cache
     *
     * @return mixed
     */
    private function cache($key, $data = false, $type = 'load')
    {
        if(isset($this->param[$key]) && $this->param[$key])
        {
            $key = $key . '_' . md5(serialize($this->param[$key]));
        }
        else
        {
            $param = Input::get();
            $key = $key . '_' . md5(serialize($param));
        }

        return Dever::cache($key, $data, 0, $type);
    }

    /**
     * config
     *
     * @return mixed
     */
    public function config($key, &$state)
    {
        if(strpos($key, 'http://') !== false)
        {
            $this->api($key, $key, '');
            $state = false;
            return;
        }
        $file = false;
        $namespace = false;
        if(strpos($key, '/') !== false)
        {
            $temp = explode('/', $key);

            $method = $temp[1];

            $file = $temp[0];
            if($file && strpos($file, '.') !== false)
            {
				$temp = explode('.', $file);
				$file = $temp[0];
				$namespace = $temp[1];
				
				# 处理后台和公开的命名空间
				if($namespace == 'manage')
				{
					//self::get('manage/auth.init');
				}
				elseif($namespace != 'public')
				{
					return;
				}
			}
            elseif(!$file && strpos($key, '-') !== false && isset(Config::$global['manage']['project']) && Config::$global['manage']['project'])
            {
                $file = Config::$global['manage']['project'];
                Config::$global['manage']['project'] = '';
            }
        }
        else
        {
            $method = $key;
        }

        $file = $file ? $file : DEVER_APP_NAME;
        $path = false;
        //$api = DEVER_APP_NAME . '_api';
        $api = 'api';

        if(isset(Config::$global['host'][$api]) && isset(Config::$global['host'][$api][$file]) && strpos(Config::$global['host'][$api][$file], 'http://') === false)
        {
            if($state == true || empty($this->data[$key]))
            {
                $path = $this->api($method, $key, Config::$global['host'][$api][$file]);
            }

            if(!$path)
            {
                $state = false;

                return;
            }
        }

        $project = Project::load($file);
        if($project)
        {
            $this->path = $path ? $path : $project['path'];
            $file = isset($project['lib']) ? $project['lib'] : $project['name'];
        }
        elseif($path)
        {
            $this->path = $path;
            $project['name'] = $file;
            $project['lang'] = $file;
        }
        else
        {
            Output::abert('project_exists', $file);
        }
        
        if(isset($this->param[$key]['CHECK']))
        {
			Api::check($method, $this->path);
		}

        $this->config[$key] = array
        (
            'method' => $method,
            'file'   => $file,
            'project' => $project['name'],
            'project_name' => $project['lang'],
            'path'   => $this->path,
            'namespace' => $namespace
        );
    }

    /**
     * initApiParam
     *
     * @return mixed
     */
    private function initApiParam(&$param)
    {
        if($page = Input::get('pg'))
        {
            $param['pg'] = $page;
        }
        if($total = Input::get('pt'))
        {
            $param['pt'] = $total;
        }
        $param['json'] = 1;
        $param['cache'] = 1;
    }

    /**
     * api 载入接口操作
     *
     * @return mixed
     */
    public function api($method, $key, $api = '')
    {
        $this->data[$key] = $this->cache($key, false, 'api');

        if($this->data[$key])
        {
            return false;
        }

        if($api && is_string($api) && strpos($api, 'http') === false)
        {
            return $api;
        }
        elseif($api && is_array($api) && isset($api[1]) && $api[1])
        {
            return $api[1];
        }
        else
        {
            if(is_array($api))
            {
                $api = $api[0];
            }
            $server = 'curl';
            if(isset(Config::$global['host']['api_server']) && isset(Config::$global['host']['api_server']['type']))
            {
                $server = Config::$global['host']['api_server']['type'];
            }

            $this->initApiParam($this->param[$key]);

            if($api && $server == 'tcp' && class_exists('\Swoole\Server'))
            {
                //$api .= 'swoole.server';

                $config = parse_url($api);

                $config['port'] = Config::$global['host']['api_server']['port'];

                $config['host'] = isset(Config::$global['host']['api_server']['host']) ? Config::$global['host']['api_server']['host'] : $config['host'];

                $class = \Dever\Server\Swoole::getInstance($config['port'], $config['host']);

                $data = $class->api($key, $this->param[$key]);

                if(!$data)
                {
                    $api .= $method;

                    $data = Dever::curl($api, $this->param[$key]);
                }
                else
                {
                    $api = $config['host'] . ':' . $config['port'] . '--' . $method;
                }
            }
            elseif($api && $server == 'rpc' && class_exists('\Yar_Server'))
            {
                $api .= 'rpc.server';

                $data = \Dever\Server\Rpc::api($api, $method, $this->param[$key]);

                $api .= '--' . $method;
            }
            else
            {
                $api .= $method;

                $data = Dever::curl($api, $this->param[$key]);
            }
        }
        

        $this->log('api', $api, $this->param[$key], $data);

        if(is_array($data))
        {
            $this->data[$key] = $data;
        }
        else
        {
            $data = json_decode($data, true);


            if(isset($data['page']))
            {
                Config::$global['page'] = $data['page'];
            }

            if(isset($data['status']) && $data['status'] == 2)
            {
                throw new \Exception($data['msg']);
                die;
            }
            elseif(isset($data['data']))
            {
                $this->data[$key] = $data['data'];
            }
            elseif(isset($data['code']) || (isset($data['status']) && $data['status'] == 2))
            {
                Dever::abert($data['msg']);
            }
            else
            {
                $this->data[$key] = $data;
            }
        }

        $this->cache($key, $this->data[$key], 'api');

        return false;
    }

    /**
     * model 载入数据与类操作
     *
     * @return string
     */
    private function import($method, $key)
    {
        if(isset($this->param[$key]) && is_array($this->param[$key]) && $this->param[$key])
        {
            foreach($this->param[$key] as $k => $v)
            {
                Input::set($k, $v);
            }
        }
        $this->data[$key] = false;

        # 以后这里可以加入方法和类引用记录
        foreach($method as $k => $v)
        {
            if(isset($this->config[$key]['method']))
            {
                $this->$v($key);

                //$this->log($v, $key, (isset($this->param[$key]) ? $this->param[$key] : $this->param[$key]), $this->data[$key]);
            }
        }
    }

    /**
     * model 载入数据库操作
     *
     * @return string
     */
    private function model($key)
    {
        if(strpos($this->config[$key]['method'], '-') !== false)
        {
            $method = explode('-', $this->config[$key]['method']);

            $index = '';
            //$class = $this->config[$key]['file']  . '-' . $method[0];
            $class = $this->config[$key]['file']  . '-' . $this->config[$key]['method'];
            if(isset($method[1]) && $method[1] && isset($method[2]) && $method[2])
            {
                $class .= '-' . $method[2];
                $index = $method[2];
            }

            if(empty($this->class[$class]))
            {
                $this->setting[$class] = array();
                
                $this->setting[$class]['name'] = $method[0];

                if(isset($this->config[$key]['base']) && isset($this->config[$key]['rel']))
                {
                    $path = DEVER_INCLUDE_PATH . $this->config[$key]['rel'];
                }
                else
                {
                    $path = $this->config[$key]['path'];
                }

                $file = $path . self::DATABASE . $method[0] . '.php';

                if(is_file($file))
                {
                    //Output::abert('file_exists', $file);

                    $this->setting[$class] = $this->file($file);
                }

                if(isset($this->setting[$class]['config']))
                {
                    Config::$global['model'][$method[0]] = $this->setting[$class]['config'];
                }

				if(isset($this->setting[$class]['top']))
				{
					if(is_string($this->setting[$class]['top']))
					{
                        if(strpos($this->setting[$class]['top'], '-') === false)
                        {
                            $this->setting[$class]['top_key'] = $this->setting[$class]['top'];

                            $this->setting[$class]['top'] = $this->config[$key]['file'] . '-' . $this->setting[$class]['top'];
                        }
						else
                        {
                            $temp = explode('-', $this->setting[$class]['top']);
                            
                            $this->setting[$class]['top_key'] = $temp[1];
                            $this->setting[$class]['top'] = $this->setting[$class]['top'];
                        }
					}
					elseif(is_array($this->setting[$class]['top']))
					{
						$this->setting[$class]['top']['key'] = $this->config[$key]['file'] . '-' . $this->setting[$class]['top']['key'];
					}
				}

                if(isset($this->setting[$class]['auth']) && is_array($this->setting[$class]['auth']))
                {
                    $this->setting[$class]['auth']['project'] = $this->config[$key]['project'];
                    $this->setting[$class]['auth']['project_name'] = $this->config[$key]['project_name'];
                }

                $this->setting[$class]['project'] = $this->config[$key]['file'];

                $this->class[$class] = new Model($this->setting[$class], $index);
            }
            
            if(isset($method[1]) && $method[1])
            {
                if(isset(Config::$global['model'][$key]))
                {
                    if(isset($this->param[$key]))
                    {
                        $this->param[$key] += Config::$global['model'][$key];
                    }
                    else
                    {
                        $this->param[$key] = Config::$global['model'][$key];
                    }
                }

                //$this->api($this->config[$key]['path'], $method[0] . '-' . $method[1]);
				if($method[1] == 'createIndex')
				{
					$this->data[$key] = $this->class[$class]->index($this->param[$key]);
				}
                elseif(isset($this->param[$key]) && $this->param[$key])
                {
                    //$this->data[$key] = call_user_func_array(array($this->class[$class], 'method'), $this->param[$key]);
                    $this->data[$key] = $this->class[$class]->method($method[1], $this->param[$key]);
                }
                else
                {
                    $this->data[$key] = $this->class[$class]->method($method[1]);
                }
            }
            elseif(isset($method[2]) && $method[2])
            {
                $this->data[$key] = $this->setting[$class][$method[2]];
            }
            else
            {
				$this->data[$key] = $this->setting[$class];
			}

            //unset($this->config[$key]['method']);
        }
    }

    /**
     * library 载入基础类库
     *
     * @return string
     */
    private function library($key)
    {
        if(strpos($this->config[$key]['method'], '.') !== false)
        {
            $method = explode('.', $this->config[$key]['method']);

            $class = ucfirst($this->config[$key]['namespace'] ? $this->config[$key]['namespace'] : $this->config[$key]['file']) . '\\' . ucfirst($method[0]);

            if(empty($this->class[$class]))
            {
                if(isset($this->config[$key]['base']) && isset($this->config[$key]['rel']))
                {
                    $path = DEVER_INCLUDE_PATH . $this->config[$key]['rel'];
                }
                else
                {
                    $path = $this->config[$key]['path'];
                }

                $file = $path . self::LIBRARY . str_replace('\\', '/', $class) . '.php';

                if(!is_file($file))
                {
                    Output::abert('file_exists', $file);
                }
                require $file;

                $className = self::NAME . $class;

                $this->data[$key] = $this->class[$class] = new $className();
            }

            if(isset($method[1]) && $method[1])
            {
                $this->call($method[1], $key, $class);
            }
            else
            {
                $this->data[$key] = $this->data[$key] ? $this->data[$key] : $this->class[$class];
            }

            //unset($this->config[$key]['method']);
        }
    }

    /**
     * service 载入业务类库
     *
     * @return string
     */
    private function service($key)
    {
        if(strpos($this->config[$key]['method'], '!') !== false)
        {
            $method = explode('!', $this->config[$key]['method']);

            $class = ucfirst($method[0]);

            # 暂时不缓存类
            $class_test = 'temp';

            if(empty($this->class[$class_test]))
            {
                if(isset($this->config[$key]['base']) && isset($this->config[$key]['rel']))
                {
                    $path = DEVER_INCLUDE_PATH . $this->config[$key]['rel'];
                }
                else
                {
                    $path = $this->config[$key]['path'];
                }

                $file = $path . self::SERVICE . $class . '.php';

                if(!is_file($file))
                {
                    Output::abert('file_exists', $file);
                }
                require_once $file;

                $className = self::SERVER_NAME . ucfirst($this->config[$key]['namespace'] ? $this->config[$key]['namespace'] : $this->config[$key]['file']) . '\\' .$class;

                $this->class[$class] = new $className();
            }

            $method[1] = 'handle';
            /*
            if(!$method[1])
            {
                $method[1] = 'handle';
            }
            */
            $this->call($method[1], $key, $class);

            //unset($this->config[$key]['method']);
        }
    }

    private function call($method, $key, $class)
    {
        $this->step($method, $class);

        if($this->param && isset($this->param[$key]))
        {
            # 以后升级实现可以多个参数的吧
            //$this->data[$key] = call_user_func_array(array($this->class[$class], $method), $this->param[$key]);
            
            $this->data[$key] = $this->class[$class]->{$method}($this->param[$key]);
        }
        else
        {
            $this->data[$key] = $this->class[$class]->{$method}();
        }
    }

    private function step($method, $class)
    {
        $key = '_step_';
        if(strpos($method, $key) != false)
        {
            $config = explode($key, $method);

            Step::init($config[0], $config[1], $key, $class);
        }
    }
    
    /**
     * 载入文件
     *
     * @return string
     */
    private function file($file)
    {
		return include($file);
	}

    /**
     * log
     *
     * @return log
     */
    private function log($type, $method, $param, $data = array())
    {
        Debug::log(array('method' => $method, 'param' => $param, 'data' => $data), $type);
    }
}