<?php namespace Dever\Security;

use Dever\Http\Input;
use Dever\Http\Output;
use Dever\Config\Load as Config;
use Dever\Config\Project;
use Dever\Routing\Load;
use Dever\Debug\Process as Debug;
use Dever\Security\Internal;

class Api
{
	/**
	 * prefix
	 *
	 * @var string
	 */
	const PREFIX = 'api_';
	
	/**
	 * time
	 *
	 * @var int
	 */
	const TIME = 300;

	/**
     * path 定义api目录
     *
     * @var string
     */
    const PATH =  'api/';
	
	/**
	 * default token
	 *
	 * @var string
	 */
	static private $token = 'dever_api';
	
	/**
	 * log
	 *
	 * @var string
	 */
	static private $log;
	
	/**
	 * state
	 *
	 * @var string
	 */
	static public $state = false;

	/**
     * init
     * @param  string  $key
     * @param  string  $path
     * 
     * @return string
     */
	static public function init($path = false)
	{
		if(!$path)
		{
			$path = DEVER_APP_PATH;
		}
		$file = $path . self::PATH . 'main.php';

		if(is_file($file))
		{
			return include($file);
		}

		return array();
	}

    static public function login($uid)
    {
        $auth = '';
        $data = array($uid, time());
        if($data)
        {
            $auth = base64_encode(Internal::encode(implode("\t", $data), self::$token));
        }

        return $auth;
    }
	
	/**
     * check
     * @param  string  $key
     * 
     * @return string
     */
	static public function check($key, $path = false)
	{
		$config = self::init($path);

		if($config && isset($config[$key]))
		{
			self::$log = array();
			self::$log['add_site'] = $key;
			
			self::$state = true;
			# 验证当前api有效性
			$request = Input::prefix(self::PREFIX);
			
			Config::$global['base']['api'] = true;

			# 增加测试效果
			/*
			if(Input::get('dever_api_test') == 'test_yes')
			{
				print_r(self::get($request));die;
			}
			*/
			
			self::result($request);
			
			if($request)
			{
				foreach($request as $k => $v)
				{
					$k = str_replace(self::PREFIX, '', $k);
					
					if(!isset($config[$key]['request'][$k]))
					{
						Output::abert('api_param_exists', $k);
					}
					
					Input::set($k, $v);
					
					self::$log['add_request'][0][$k] = $v;
				}
			}
		}
	}
	
	/**
     * 得到数据结果
     * 
     * @return mixed
     */
	static public function out($data, $state = false)
	{
		if(self::$state == true || $state == true)
		{
			if(!$data)
			{
				return;
				//Output::abert('api_param_exists', 'response');
			}
			
			# 记录日志 暂时不做记录
			if(Project::load('manage'))
			{
				self::$log['add_response'][0] = $data;
				//Load::get('manage/api_log-insert', $param);
			}
			
			$result['msg'] = 'success';
			$result['status'] = 1;
			$result['data'] = $data;
			
			if(Input::get('dever_api_data') == 'yes')
			{
				print_r($result);die;
			}
			Output::result($result);
		}

		Debug::out();
	}
	
	/**
     * 得到解密结果
     * 
     * @return mixed
     */
	static public function result($info)
	{
		$time 	= Input::get('time');
		if(!$time)
		{
			return self::resultLogin();
		}
		# 验证时间是否超时，默认为5分钟
		if(time() - $time > self::TIME)
		{
			Output::abert('api_signature_exists');
		}
		$nonce 	= Input::get('nonce');
		$key 	= Input::get('signature');
		$signature = self::signature($time, $nonce, $info);
		
		if(Input::get('signature') != $signature)
		{
			Output::abert('api_signature_exists');
		}

		return $signature;
	}

	/**
     * 得到解密结果
     * 
     * @return mixed
     */
	static public function resultLogin()
    {
    	$signature 	= Input::get('signature');
        $auth     	= array();
        $user 		= array();
        if($signature)
        {
            $auth = explode("\t", Internal::decode(base64_decode($signature), self::$token));
        }

        list($uid, $time) = (empty($auth) || count($auth) < 2) ? array(0, '') : $auth;

        if(!empty($uid) && (time() - $time) < 2592000)
        {
            $user = array('uid' => $uid,'time'=>$time);
            Input::set('uid', $uid);
            return $user;
        }
        else
        {
            Output::abert('api_signature_exists');
        }
    }
	
	/**
     * 得到签名及其数据
     * 
     * @return mixed
     */
	static public function get($info)
	{
		$time 	= time();
		$nonce 	= self::nonce();
		$signature = self::signature($time, $nonce, $info);
		
		$info += array
		(
			'time'	=> $time,
			'nonce'	=> $nonce,
			'signature'	=> $signature,
			//'token' => self::token(),
			//'status'	=> 1,
			//'msg'		=> 'success',
		);
		
		return $info;
	}
	
	/**
     * 获取signature
     * 
     * @return mixed
     */
    static public function signature($time, $nonce, $info = array())
    {
		$info['token'] = self::token();
		$info['time'] = $time;
		$info['nonce'] = $nonce;
		ksort($info);

		$signature_string = '';
		foreach($info as $k => $v)
		{
			$signature_string .= $k . '=' . $v . '&';
		}
		$signature_string = substr($signature_string, 0, -1);
		return sha1($signature_string);
	}
	
	/**
     * 获取token
     * 
     * @return mixed
     */
	static public function token()
	{
		self::$token = isset(Config::$global['base']['token']) ? Config::$global['base']['token'] : self::$token;
		return md5(self::$token);
	}
	
	/**
     * 获取nonce
     * 
     * @return mixed
     */
	static public function nonce()
	{
		return substr(md5(microtime()), rand(10, 15));
	}
}