<?php namespace Dever\Template;

use Dever;
use Dever\Http\Input;
use Dever\Http\Output;
use Dever\Session\Save;
use Dever\Config\Load as Config;
use Dever\Config\Project;

class View
{
	/**
	 * templatePath
	 *
	 * @var string
	 */
	const TEMPLATE = 'html';

	/**
	 * servicePath
	 *
	 * @var string
	 */
	const SERVICE =  'template';
	

	/**
	 * template
	 *
	 * @var string
	 */
	protected $template;

	/**
	 * template
	 *
	 * @var string
	 */
	protected $service;

	/**
	 * method
	 *
	 * @var string
	 */
	protected $method;

	/**
	 * content
	 *
	 * @var string
	 */
	protected $content;

	/**
	 * path
	 *
	 * @var string
	 */
	protected $path;

	/**
	 * parse (default value is dom)
	 *
	 * @var string
	 */
	protected $parse = 'Dom';

	/**
	 * file
	 *
	 * @var string
	 */
	protected $file;

	/**
	 * data
	 *
	 * @var array
	 */
	protected $data = NULL;

	/**
	 * pjax
	 *
	 * @var bool
	 */
	protected $pjax = NULL;

	/**
	 * project
	 *
	 * @var string
	 */
	protected $project;

	/**
	 * project
	 *
	 * @var string
	 */
	protected $porject_path;

	/**
	 * assets
	 *
	 * @var string
	 */
	protected $assets;

	/**
	 * compile
	 *
	 * @var \Dever\Template\Compile
	 */
	protected $compile;

	/**
     * instance
     *
     * @var string
     */
    static protected $instance;

    /**
     * load file
     * @param string $file
     * @param string $path
     * 
     * @return \Dever\Template\View
     */
    static public function getInstance($file, $path = '', $project = false)
    {
    	$key = $path . DIRECTORY_SEPARATOR . $file;
        if(empty(self::$instance[$key]))
        {
            self::$instance[$key] = new self($file, $path);

            self::$instance[$key]->project($project);
        }

        return self::$instance[$key];
    }
	
	/**
	 * __construct
	 * @param string $file
	 * @param string $path
	 *
	 * @return mixed
	 */
	public function __construct($file, $path = '')
	{
		$this->file = $file;

		$this->path($path);

		$this->content = '';
	}

	/**
	 * project
	 * @param string $project
	 * @param array $data
	 * 
	 * @return mixed
	 */
	public function project($project, $data = array())
	{
		$this->assets = Config::$global['base']['assets'];
		$this->service = DEVER_APP_PATH;
		if($data) $this->data = $data;

		$this->project = $project;

		if($this->project)
		{
			$project = Project::load($this->project);
			$this->assets = str_replace(DEVER_APP_PATH, $project['path'], $this->assets);
			$this->service = $project['path'];
		}

		$this->service .=  self::SERVICE .  DIRECTORY_SEPARATOR;
	}

	/**
	 * page
	 * @param string $service
	 * 
	 * @return \Dever\Template\View
	 */
	public function page($file, $path = '')
	{
		echo $this->load($file, $path);
	}

	/**
	 * load service
	 * @param string $service
	 * 
	 * @return \Dever\Template\View
	 */
	public function load($file, $path = '', $project = false)
	{
		$view = View::getInstance($file, $path, $project);

		return $view->runing();
	}

	/**
	 * load service
	 * @param string $service
	 * 
	 * @return \Dever\Template\View
	 */
	static public function get($file, $path = '', $project = false)
	{
		$class = new self($file, $path);

        $class->project($project);

        return $class->runing();

        /*
		$view = View::getInstance($file, $path, $project);

		return $view->runing();
		*/
	}

	/**
	 * html 主流模板引擎的方式
	 * @param $file 文件名
	 * @param $data 数据
	 *
	 * @return mixed
	 */
	static public function html($file = false, $data = array(), $project = false)
	{
		$class = new self($file, '');

		$class->parse = false;

		$class->project($project, $data);

        return $class->runing();
	}

	/**
	 * path
	 * @param string $path
	 * 
	 * @return \Dever\Template\View
	 */
	public function path($path = false)
	{
		if(isset(Config::$global['base']['template']))
		{
			$save = new Save();
			$template = Input::get(Config::$global['base']['template']);
			if($template)
			{
				if($template == 'none')
				{
					$save->un(Config::$global['base']['template']);
				}
				else
				{
					$save->add(Config::$global['base']['template'], $template);
				}
			}

			$template = $save->get(Config::$global['base']['template']);
		}
		
		if(isset($template) && $template)
		{
			$this->path = $template . DIRECTORY_SEPARATOR;
		}
		elseif($path)
		{
			if($path == 'manage')
			{
				Config::$global['base']['service'] = 'manage';
			}
			$this->path = $path . DIRECTORY_SEPARATOR;
		}
		elseif(isset(Config::$global['base']['template']))
		{
			if(is_array(Config::$global['base']['template']))
			{
				$this->path = Dever::mobile() ? Config::$global['base']['template'][1] : Config::$global['base']['template'][0];
			}
			else
			{
				$this->path = Config::$global['base']['template'];
			}
			$this->path .= DIRECTORY_SEPARATOR;
		}
		else
		{
			$temp = explode(DIRECTORY_SEPARATOR, DEVER_APP_PATH);
			$this->path  = $temp[count($temp)-2] . DIRECTORY_SEPARATOR;
		}

		return $this;
	}

	/**
	 * parse
	 * @param string $parse
	 * 
	 * @return \Dever\Template\View
	 */
	public function parse($parse)
	{
		$parse = 'Dever\\Template\\Parse\\' . ucwords($parse);

		if(class_exists($parse))
		{
			$this->parse = new $parse($this->compile);
		}

		return $this;
	}

	/**
	 * runing
	 *
	 * @return \Dever\Template\View
	 */
	public function runing()
	{
		$key = \Dever\Routing\Uri::url();
		$this->content = $this->cache($key);
		if($this->content)
		{
			return $this->content;
		}

		$this->template()->compile();
		$this->cache($key, $this->content);

		return $this->content;
	}

	/**
	 * file
	 *
	 * @return string
	 */
	public function file()
	{
		if(empty($this->compile))
		{
			return '';
		}
		return $this->compile->file();
	}

	/**
	 * compile
	 *
	 * @return \Dever\Template\View
	 */
	public function compile()
	{
		$this->layout();

		$this->compile = new Compile($this->file, $this->template, $this->service, $this->project, $this->path, $this->data);

		$this->content = $this->compile->get();

		if($this->content || $this->content !== false)
		{
			return $this->content;
		}

		if(!$this->parse)
		{
			return $this->display();
		}

		return $this->service();
	}

	/**
	 * layout
	 *
	 * @return mixed
	 */
	public function layout()
	{
		if(isset(Config::$global['base']['layout']) && empty($this->pjax))
		{
			if(array_key_exists('HTTP_X_PJAX', $_SERVER) && $_SERVER['HTTP_X_PJAX'])
			{
				$this->pjax = true;
			}
			else
			{
				$this->pjax = false;
			}
		}
	}

	/**
	 * service
	 *
	 * @return \Dever\Template\View
	 */
	public function service()
	{
		if(!is_file($this->service))
		{
			return $this->display();
		}
		$view = $this;

		require $this->service;

		return $this->content;
	}

	/**
	 * template
	 *
	 * @return \Dever\Template\View
	 */
	public function template($state = 1)
	{
		//$path = str_replace(DEVER_APP_PATH, '', $this->assets);
		//$path = $path == $this->assets ? str_replace(DEVER_PATH, '', $this->assets) : $path;
		if($this->path && strpos($this->assets, DIRECTORY_SEPARATOR . $this->path) !== false)
		{
			$this->path = '';
		}

		$html = isset(Config::$global['base']['html']) ? Config::$global['base']['html'] . DIRECTORY_SEPARATOR : self::TEMPLATE . DIRECTORY_SEPARATOR; 

		$this->template = $this->assets  . $this->path . $html;

		if($state == 1 && !is_file($this->template . $this->file . '.html') && isset(Config::$global['base']['template']) && is_array(Config::$global['base']['template']) && isset(Config::$global['base']['template'][0]))
		{
			# 当前文件不存在，则跳转
			$this->path(Config::$global['base']['template'][0] . DIRECTORY_SEPARATOR);
			
			return $this->template(2);
		}
		
		//echo Config::$global['host']['css'];die;

		/*
		if($this->path && (DEVER_APP_LIB . DIRECTORY_SEPARATOR == $this->path || DEVER_APP_NAME . DIRECTORY_SEPARATOR == $this->path))
		{
			$this->path = '';
		}
		*/

		$this->sername = (isset(Config::$global['base']['service']) && $this->path) ? Config::$global['base']['service'] . DIRECTORY_SEPARATOR : $this->path;

		$this->service 	= $this->service . $this->sername . $this->file . '.php';

		//echo $this->service . "\r\n";

		$this->replace();
		
		return $this;
	}

	private function replace()
	{
		if($this->path && empty(Config::$global['host']['replace']))
		{
			if(isset(Config::$global['base']['replace']) && is_array(Config::$global['base']['replace']))
			{
				foreach(Config::$global['base']['replace'] as $k => $v)
				{
					$this->replaceOne($k);
				}
			}

	        Config::$global['host']['replace'] = true;
		}
	}

	private function replaceOne($k)
	{
		if($k == 'public')
		{
			return;
		}
		$key = 'assets/';

		if(isset(Config::$global['base']['template']) && is_array(Config::$global['base']['template']))
		{
			foreach(Config::$global['base']['template'] as $v)
			{
				if(isset(Config::$global['host'][$k]) && strpos(Config::$global['host'][$k], $key . $v) !== false)
				{
					return;
				}
			}
		}

		if(isset(Config::$global['host'][$k]))
		{
			Config::$global['host'][$k] = str_replace($key, $key . $this->path, Config::$global['host'][$k]);
		}
	}

	/**
	 * fetch
	 *
	 * @return \Dever\Template\View
	 */
	public function fetch()
	{
		$this->method[] = func_get_args();

		return $this;
	}

	/**
	 * check
	 *
	 * @return \Dever\Template\View
	 */
	public function check()
	{
		$this->method[] = func_get_args();

		return $this;
	}

	/**
	 * set　设置变量
	 * @param $key 变量名
	 * @param $value 变量的值
	 *
	 * @return \Dever\Template\View
	 */
	public function set($key, $value = false)
	{
		$this->compile->set($key, $value);

		return $this;
	}

	/**
	 * hook
	 * @param $key
	 *
	 * @return \Dever\Template\View
	 */
	public function hook($key, $value)
	{
		$this->compile->set('Dever::$global[\'hook\'][\'' . $key. '\']', $value);

		$value = '<{function(){echo 1;}}>';
		$this->fetch($key, $value);

		return $this;
	}

	/**
	 * display
	 * @param $file 文件名
	 * @param $local 是否本地化 默认不抓取文件到本地
	 *
	 * @return mixed
	 */
	public function display($file = false, $local = false)
	{
		if($file)
		{
			$this->compile->read($file, $this->template, $this->service, $local);
		}

		if($this->method)
		{
			if(is_string($this->parse))
			{
				$this->parse($this->parse);
			}

			if(is_object($this->parse))
			{
				$node = array();

				if(isset(Config::$global['base']['data']))
		        {
		            $path = Config::$global['base']['data'];
		        }
		        else
		        {
		            $path = DEVER_PATH . 'data/';
		        }
				$file = $path . 'node/' . DEVER_APP_NAME . '_' . trim($this->sername, DIRECTORY_SEPARATOR) . '.php';
				if(is_file($file))
				{
					$node = include($file);
				}
				
				# 为了更好的兼容大部分php配置，取消这里的匿名函数...
				foreach($this->method as $k => $v)
				{
					$this->parse->make($v);
					# 节点记录
					//$this->node($node, $v);
				}

				if($node)
				{
					file_put_contents($file, '<?php return ' . var_export($node, true) . ';');
				}
				
				/*
				$callback = function($param, $key)
				{
					$this->parse->make($param);
				};

				array_walk($this->method, $callback);
				*/

				$this->content = $this->compile->create($this->parse->get());
			}
		}
		else
		{
			$this->content = $this->compile->create($this->content);
		}

		return $this->content;
	}
	
	private function node(&$node, $param = array())
	{
		$config = array('none', 'null', 'remove');
		if((isset($param[2]) && is_string($param[2]) && !array_search($param[2], $config)) || (isset($param[3]) && is_string($param[3])))
		{
			if(is_array($param[0]))
			{
				$param[0] = implode(',', $param[0]);
			}
			
			$node[$this->file][$param[0]] = $param;
		}
	}

	/**
	 * result 获取当前视图模板的基本信息
	 *
	 * @return array
	 */
	public function result()
	{
		$result['service'] = $this->service;
		$result['template'] = $this->template . $this->file . '.html';
		$result['cmp'] = $this->compile->file();

		//$content = file_get_contents($result['template']);

		//$rule = '<script class="include"(.*?)path="(.*?)"(.*?)file="(.*?)"(.*?)><\/script>';

        //preg_match_all(DIRECTORY_SEPARATOR . $rule . '/i', $content, $result);
		
		return $result;
	}

	private function cache($key, $data = false)
    {
        return Dever::cache($key, $data, 0, 'html');
    }
}